package keeper

import (
	"context"
	"errors"
	"fmt"
	"math/big"
	"strconv"

	"cosmos-test/x/cosmostest/auctionconfig"
	"cosmos-test/x/cosmostest/types"

	sdk "github.com/cosmos/cosmos-sdk/types"
)

func (k msgServer) NewAuction(goCtx context.Context, msg *types.MsgNewAuction) (*types.MsgNewAuctionResponse, error) {
	ctx := sdk.UnwrapSDKContext(goCtx)

	next, found := k.Keeper.GetNextAuction(ctx)
	if !found {
		return nil, errors.New("Auction not found")
	}
	index := strconv.FormatUint(next.AuctionId, 10)

	auction := types.Auction{
		Index:       index,
		Name:        msg.Name,
		Description: msg.Description,
		// Best:        new(types.Bid),
		Deadline:   uint64(ctx.BlockHeight()) + auctionconfig.AuctionTime,
		Denom:      msg.Denom,
		Owner:      msg.Creator,
		Ceiling:    msg.Ceiling,
		LeaseStart: uint64(ctx.BlockTime().Unix()),
		LeaseEnd:   msg.LeaseEnd,
		Closed:     false,
		Remaining:  "0",
	}

	senderAddr, err := sdk.AccAddressFromBech32(msg.Creator)
	if err != nil {
		return nil, fmt.Errorf("sender address `%s` format invalid (bech32 required)", msg.Creator)
	}

	spendable := k.bank.SpendableCoins(ctx, senderAddr)
	// if balance does not exceed or equal proposed auction ceiling...
	ceiling := new(big.Int)
	ceiling.SetString(auction.Ceiling, 10)
	if spendable.AmountOf(auction.Denom).BigInt().Cmp(ceiling) == -1 {
		return nil, fmt.Errorf("not enough balance to set ceiling %s%s", msg.Ceiling, auction.Denom)
	}

	coins := sdk.NewCoins(sdk.Coin{
		Amount: spendable.AmountOf(auction.Ceiling),
		Denom:  auction.Denom,
	})

	if err := k.bank.SendCoinsFromAccountToModule(ctx, senderAddr, "cosmostest", coins); err != nil {
		return nil, fmt.Errorf("failed to transfer %s%s", auction.Ceiling, auction.Denom)
	}

	k.Keeper.SetAuction(ctx, auction)
	next.AuctionId++

	k.Keeper.SetNextAuction(ctx, types.NextAuction{AuctionId: next.AuctionId})

	return &types.MsgNewAuctionResponse{
		AuctionId: strconv.FormatUint(next.AuctionId, 10),
	}, nil
}
